package tech.nevets.modbot;

import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.Role;
import net.dv8tion.jda.api.events.interaction.SlashCommandEvent;
import net.dv8tion.jda.api.events.message.guild.GuildMessageReceivedEvent;
import tech.nevets.modbot.api.commands.*;

import java.util.Arrays;
import java.util.List;
import java.util.regex.Pattern;

public class CommandManager {


    public boolean hasPermission(Member member, ICommand cmd) {
        List<Role> memberRoles = member.getRoles();
        int permLevel = 0;

        for (Role memberRole : memberRoles) {
            switch (memberRole.getName().toLowerCase()) {
                case "helper", "1":
                    if (permLevel < 1) permLevel = 1;
                    continue;
                case "moderator", "mod", "2":
                    if (permLevel < 2) permLevel = 2;
                    continue;
                case "administrator", "admin", "3":
                    if (permLevel < 3) permLevel = 3;
                    continue;
                case "owner", "4":
                    if (permLevel < 4) permLevel = 4;
            }
        }
        return permLevel >= cmd.getPermissionLevel();
    }

    public void handle(GuildMessageReceivedEvent event) {
        String[] split = event.getMessage().getContentRaw()
                .replaceFirst("(?i)" + Pattern.quote(CoreBot.CORE_CONFIG.getConfig().getString("bot.prefix")), "")
                .split("\\s+");

        String invoke = split[0].toLowerCase();
        IPrefixCommand cmd = Registry.getPrefixCommand(invoke);

        assert event.getMember() != null;
        assert cmd != null;
        if (!hasPermission(event.getMember(), cmd)) {
            event.getMessage().reply("You do not have permission to use this command.").queue();
            return;
        }

        event.getChannel().sendTyping().queue();
        List<String> args = Arrays.asList(split).subList(1, split.length);

        CommandContext ctx = new CommandContext(event, args);

        cmd.handle(ctx);
    }

    public void handle(SlashCommandEvent event) {
        String invoke = event.getName();
        ISlashCommand cmd = Registry.getSlashCommand(invoke);

        if (cmd != null) {
            event.deferReply().queue();

            assert event.getMember() != null;
            if (!hasPermission(event.getMember(), cmd)) {
                event.getHook().sendMessage("You do not have permission to use this command.").queue();
                return;
            }

            CommandContext ctx = new CommandContext(event);

            cmd.handleSlash(ctx);
        }
    }
}
