package tech.nevets.modbot;

import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.Role;
import net.dv8tion.jda.api.events.interaction.SlashCommandEvent;
import net.dv8tion.jda.api.events.message.guild.GuildMessageReceivedEvent;
import net.dv8tion.jda.api.hooks.ListenerAdapter;
import net.dv8tion.jda.api.interactions.commands.build.CommandData;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import tech.nevets.modbot.api.commands.CommandContext;
import tech.nevets.modbot.api.commands.ICommand;
import tech.nevets.modbot.api.commands.IPrefixCommand;
import tech.nevets.modbot.api.commands.ISlashCommand;

import javax.annotation.Nullable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.regex.Pattern;

public class Registry {
    private static final Logger LOGGER = LoggerFactory.getLogger(Registry.class);

    public static final List<ICommand> COMMANDS = new ArrayList<>();
    public static final List<IPrefixCommand> PREFIX_COMMANDS = new ArrayList<>();
    public static final List<ISlashCommand> SLASH_COMMANDS = new ArrayList<>();
    private static final EventExecutor EVENT_EXECUTOR = new EventExecutor();

    @Nullable
    public static ICommand getCommand(String search) {
        String searchLower = search.toLowerCase();

        for (ICommand cmd : COMMANDS) {
            if (cmd.getName().equals(searchLower) || cmd.getAliases().contains(searchLower)) {
                return cmd;
            }
        }
        return null;
    }

    @Nullable
    public static IPrefixCommand getPrefixCommand(String search) {
        String searchLower = search.toLowerCase();

        for (IPrefixCommand cmd : PREFIX_COMMANDS) {
            if (cmd.getName().equals(searchLower) || cmd.getAliases().contains(searchLower)) {
                return cmd;
            }
        }
        LOGGER.warn("Returned null for command search: " + searchLower);
        return null;
    }

    @Nullable
    public static ISlashCommand getSlashCommand(String search) {
        String searchLower = search.toLowerCase();

        for (ISlashCommand cmd : SLASH_COMMANDS) {
            if (cmd.getName().equals(searchLower) || cmd.getAliases().contains(searchLower)) {
                return cmd;
            }
        }
        LOGGER.warn("Returned null for command search: " + searchLower);
        return null;
    }

    public static List<ICommand> getCommands() {
        return COMMANDS;
    }

    public static List<IPrefixCommand> getPrefixCommands() {
        return PREFIX_COMMANDS;
    }

    public static List<ISlashCommand> getSlashCommands() {
        return SLASH_COMMANDS;
    }

    public static EventExecutor getListeners() {
        return EVENT_EXECUTOR;
    }

    public static boolean exists(ICommand cmd) {
        boolean nameExists = COMMANDS.stream().anyMatch(it -> it.getName().equalsIgnoreCase(cmd.getName()));
        if (nameExists) {
            throw new IllegalArgumentException("A command with that name already exists!");
        }

        return false;
    }

    public static void register(IPrefixCommand cmd) {
        if (!exists(cmd)) {
            COMMANDS.add(cmd);
            PREFIX_COMMANDS.add(cmd);
        }
    }

    public static void register(ISlashCommand cmd) {
        if (!exists(cmd)) {
            COMMANDS.add(cmd);
            SLASH_COMMANDS.add(cmd);
        }
    }

    public static void register(ListenerAdapter listener) {
        EVENT_EXECUTOR.registerListener(listener);
    }

    public static void loadSlashCommands() {
        if (SLASH_COMMANDS.size() > 0) {
            List<CommandData> cmdDataList = new ArrayList<>();
            for (ISlashCommand slashCmd : SLASH_COMMANDS) {
                cmdDataList.add(slashCmd.getCommandData());
            }
            CoreBot.loadSlashCommands(cmdDataList);
        }
    }
}
