package tech.nevets.modbot.api.commands;

import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.Role;
import net.dv8tion.jda.api.events.interaction.SlashCommandEvent;
import net.dv8tion.jda.api.events.message.guild.GuildMessageReceivedEvent;
import net.dv8tion.jda.api.interactions.commands.build.CommandData;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import tech.nevets.modbot.CoreBot;

import javax.annotation.Nullable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.regex.Pattern;

public class CommandRegistry {
    private static final Logger LOGGER = LoggerFactory.getLogger(CommandRegistry.class);
    private final List<ICommand> commands = new ArrayList<>();
    private final List<IPrefixCommand> prefixCommands = new ArrayList<>();
    private final List<ISlashCommand> slashCommands = new ArrayList<>();

    public CommandRegistry() {}

    @Nullable
    public IPrefixCommand getPrefixCommand(String search) {
        String searchLower = search.toLowerCase();

        for (IPrefixCommand cmd : prefixCommands) {
            if (cmd.getName().equals(searchLower) || cmd.getAliases().contains(searchLower)) {
                return cmd;
            }
        }
        LOGGER.warn("Returned null for command search: " + searchLower);
        return null;
    }

    @Nullable
    public ISlashCommand getSlashCommand(String search) {
        String searchLower = search.toLowerCase();

        for (ISlashCommand cmd : slashCommands) {
            if (cmd.getName().equals(searchLower) || cmd.getAliases().contains(searchLower)) {
                return cmd;
            }
        }
        return null;
    }

    public List<ICommand> getCommands() {
        return commands;
    }

    public List<IPrefixCommand> getPrefixCommands() {
        return prefixCommands;
    }

    public List<ISlashCommand> getSlashCommands() {
        return slashCommands;
    }

    public boolean hasPermission(Member member, ICommand cmd) {
        List<Role> memberRoles = member.getRoles();
        int permLevel = 0;

        for (Role memberRole : memberRoles) {
            switch (memberRole.getName().toLowerCase()) {
                case "helper", "1":
                    if (permLevel < 1) permLevel = 1;
                    continue;
                case "moderator", "mod", "2":
                    if (permLevel < 2) permLevel = 2;
                    continue;
                case "administrator", "admin", "3":
                    if (permLevel < 3) permLevel = 3;
                    continue;
                case "owner", "4":
                    if (permLevel < 4) permLevel = 4;
            }
        }
        return permLevel >= cmd.getPermissionLevel();
    }


}
